package inc::Module::JSAN::DSL;

require inc::Module::Build::Functions::DSL;
require Module::JSAN::DSL;

__PACKAGE__;

__END__

=pod

=head1 NAME

inc::Module::JSAN::DSL - Module::JSAN configuration system with DSL support

=head1 SYNOPSIS

  use inc::Module::JSAN::DSL;

=head1 DESCRIPTION

This is a loader module for Module::JSAN::DSL. It doesn't provide
any functionality by itself, please refer to L<Module::JSAN::DSL>
documentation for a description of DSL and to L<Module::JSAN> for 
general information about JSAN distributions.  


=head1 AUTHORS

Nickolay Platonov, C<< <nplatonov at cpan.org> >>

=head1 ACKNOWLEDGEMENTS

Many thanks to Module::Install authors, on top of which this module is mostly based.

=head1 COPYRIGHT

Copyright 2009 Nickolay Platonov, C<< <nplatonov at cpan.org> >>

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

See L<http://www.perl.com/perl/misc/Artistic.html>

=cut
